from . import kicks, pop, events, classify, obs, plot, sfh, utils, hydro
from ._version import __version__
from .citations import CITATIONS

__bibtex__ = __citation__ = CITATIONS["general"]["cogsworth"]["bibtex"]
__uri__ = "https://cogsworth.readthedocs.io/"
__author__ = "Tom Wagg"
__email__ = "tomjwagg@gmail.com"
